import pkg_resources
from werkzeug.exceptions import MethodNotAllowed

from flaschengeist.hook import HookCall

send_message_hook = HookCall("send_message")
"""Hook for sending messages, register to send the message
Args:
    message: Message object to send
"""

update_user_hook = HookCall("update_user")
"""When ever an user update is done, this is called before.
Args:
    user: User object
"""


class Plugin:
    """Base class for all Plugins"""
    def __init__(self, config=None, blueprint=None, permissions=[]):
        """Constructor called by create_app
        Args:
            config: Dict configuration containing the plugin section
            blueprint: A flask blueprint containing all plugin routes
            permissions: List of permissions of this Plugin
        """
        self.blueprint = blueprint
        self.permissions = permissions
        self.version = pkg_resources.get_distribution(self.__module__.split(".")[0]).version

    def install(self):
        """Installation routine
        Is always called with Flask application context
        """
        pass

    def serialize(self):
        """Serialize a plugin into a dict

        Returns:
            Dict containing version and permissions of the plugin
        """
        return {"version": self.version, "permissions": self.permissions}


class AuthPlugin(Plugin):
    def login(self, user, pw):
        """Login routine, MUST BE IMPLEMENTED!

        Args:
            user: User class containing at least the uid
            pw: given password
        Returns:
            Must return False if not found or invalid credentials, True if success
        """
        raise NotImplemented

    def update_user(self, user):
        """If backend is using external data, then update this user instance with external data
        Args:
            user: User object
        """
        pass

    def modify_user(self, user, password, new_password=None):
        """If backend is using (writeable) external data, then update the external database with the user provided.

        Args:
            user: User object
            password: Password (some backends need the current password for changes) if None force edit (admin)
            new_password: If set a password change is requested
        Raises:
            NotImplemented: If backend does not support this feature (or no password change)
            BadRequest: Logic error, e.g. password is wrong.
            Error: Other errors if backend went mad (are not handled and will result in a 500 error)
        """
        raise NotImplemented

    def create_user(self, user, password):
        """If backend is using (writeable) external data, then create a new user on the external database.

        Args:
            user: User object
            password: string

        """
        raise MethodNotAllowed

    def delete_user(self, user):
        """If backend is using (writeable) external data, then delete the user from external database.

        Args:
            user: User object

        """
        raise MethodNotAllowed
