from flaschengeist.models.user import User
from sqlalchemy import func
from datetime import datetime

from werkzeug.exceptions import BadRequest

from flaschengeist import logger
from flaschengeist.database import db

from .models import Transaction
from . import permissions


__attribute_limit = "balance_limit"


def set_limit(user: User, limit: float, override=True):
    if override or not user.has_attribute(__attribute_limit):
        user.set_attribute(__attribute_limit, limit)
        db.session.commit()


def get_limit(user: User) -> float:
    return user.get_attribute(__attribute_limit, default=None)


def get(user, start: datetime, end: datetime):
    credit = (
        db.session.query(func.sum(Transaction.amount))
        .filter(Transaction.receiver == user)
        .filter(start <= Transaction.time)
        .filter(Transaction.time <= end)
        .scalar()
    ) or 0
    logger.debug(credit)
    if credit is None:
        credit = 0
    debit = (
        db.session.query(func.sum(Transaction.amount))
        .filter(Transaction.sender == user and start <= Transaction.time <= end)
        .all()[0][0]
    )
    if debit is None:
        debit = 0
    return credit, debit, credit - debit


def send(sender: User, receiver, amount: float, author: User):
    """Send credit from one user to an other

    Args:
        sender: User who sends the amount
        receiver: User who receives the amount
        amount: Amount to send
        author: User authoring this transaction
    Raises:
        BadRequest if amount <= 0
    """
    if amount <= 0:
        raise BadRequest

    if sender.has_attribute(__attribute_limit):
        if (get(sender)[2] - amount) < sender.get_attribute(__attribute_limit) and not author.has_permission(
            permissions.EXCEED_LIMIT
        ):
            raise BadRequest("Limit exceeded")

    transaction = Transaction(sender=sender, receiver=receiver, amount=amount, author=author)
    db.session.add(transaction)
    db.session.commit()


def change_balance(user, amount: float, author):
    """Change balance of user

    Args:
        user: User to change balance
        amount: Amount to change balance
        author: User authoring this transaction
    """
    sender = user if amount < 0 else None
    receiver = user if amount > 0 else None
    send(sender, receiver, abs(amount), author)
